package com.sd.business.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.core.FlowThreadLocalUtil;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.apply.dto.ApplyBuyDto;
import com.sd.business.entity.apply.po.ApplyBuy;
import com.sd.business.entity.apply.po.ApplyBuyBom;
import com.sd.business.service.apply.ApplyBuyService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 发起申购流程
 */
@Service
public class ApplyBuyFlow extends FlowDelegate {

    @Autowired
    private ApplyBuyService applyBuyService;

    @Override
    public String getFlowKey() {
        return "apply_buy";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {

        ApplyBuyDto applyBuyDto = submitData.toJavaObject(ApplyBuyDto.class);
        validated(applyBuyDto);

        applyBuyDto.setFlowId(flowId);
        applyBuyDto.setFlowStatus(FlowStatusEnum.IN_PROGRESS.getKey());

        if (applyBuyDto.getId() == null) {
            applyBuyService.add(applyBuyDto);
        } else {
            applyBuyService.edit(applyBuyDto);
        }

        return applyBuyDto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        applyBuyService.update(q -> q
                .eq(BaseIdPo::getId, businessId)
                .set(ApplyBuy::getFlowStatus, FlowStatusEnum.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        applyBuyService.addBomAccessory(businessId);
    }

    /**
     * 流程退回到发起人
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    @Override
    public void returnToOriginator(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        applyBuyService.update(q -> q
                .eq(BaseIdPo::getId, FlowThreadLocalUtil.getBusinessId())
                .set(ApplyBuy::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    /**
     * 流程重新发起
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     * @param submitData 发起参数
     */
    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        ApplyBuyDto applyBuyDto = submitData.toJavaObject(ApplyBuyDto.class);
        validated(applyBuyDto);
        applyBuyDto.setFlowStatus(FlowStatusEnum.IN_PROGRESS.getKey());
        applyBuyService.edit(applyBuyDto);
    }

    /**
     * 流程驳回
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        applyBuyService.update(q -> q
                .eq(BaseIdPo::getId, FlowThreadLocalUtil.getBusinessId())
                .set(ApplyBuy::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    /**
     * 作废
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        reject(flowId, businessId, flowStatus);
    }

    /**
     * 验证数据
     */
    private void validated(ApplyBuyDto applyBuyDto) {
        Assert.notBlank(applyBuyDto.getApplyName(), "申购人不能为空");
        Assert.notEmpty(applyBuyDto.getApplyTime(), "申购时间不能为空");

        List<ApplyBuyBom> applyBuyBomList = applyBuyDto.getApplyBuyBomList();
        Assert.notEmpty(applyBuyBomList, "申购清单不能为空");

        for (ApplyBuyBom applyBuyBom : applyBuyBomList) {
            Assert.notNull(applyBuyBom.getBomSpecId(), "bom规格Id不能为空");
            Assert.notNull(applyBuyBom.getQuantity(), "申购数量不能为空");
            Assert.gtZero(applyBuyBom.getQuantity(), "申购数量必须大于0");
        }
    }

}
