package com.sd.business.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.core.FlowThreadLocalUtil;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.purchase.dto.PurchaseReturnDto;
import com.sd.business.entity.purchase.po.PurchaseReturn;
import com.sd.business.entity.purchase.po.PurchaseReturnBom;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.purchase.PurchaseReturnBomService;
import com.sd.business.service.purchase.PurchaseReturnService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 发起采购退货流程
 */
@Service
public class PurchaseReturnFlow extends FlowDelegate {

    @Autowired
    private PurchaseReturnService purchaseReturnService;

    @Autowired
    private PurchaseReturnBomService purchaseReturnBomService;

    @Autowired
    private InventoryService inventoryService;

    @Override
    public String getFlowKey() {
        return "purchase_return";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        PurchaseReturnDto purchaseReturn = submitData.toJavaObject(PurchaseReturnDto.class);
        validated(purchaseReturn);
        purchaseReturn.setFlowStatus(FlowStatusEnum.IN_PROGRESS.getKey());
        purchaseReturn.setFlowId(flowId);

        if (purchaseReturn.getId() == null) {
            purchaseReturnService.add(purchaseReturn);
        } else {
            purchaseReturnService.edit(purchaseReturn);
        }

        return purchaseReturn.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        purchaseReturnService.update(q -> q
                .eq(BaseIdPo::getId, businessId)
                .set(PurchaseReturn::getFlowStatus, FlowStatusEnum.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        // 出库采购退货
        purchaseReturnService.outPurchaseReturn(businessId);
    }

    /**
     * 流程退回到发起人
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    @Override
    public void returnToOriginator(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        purchaseReturnService.update(q -> q
                .eq(BaseIdPo::getId, businessId)
                .set(PurchaseReturn::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    /**
     * 流程重新发起
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     * @param submitData 发起参数
     */
    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        PurchaseReturnDto purchaseReturn = submitData.toJavaObject(PurchaseReturnDto.class);
        validated(purchaseReturn);
        purchaseReturnService.update(q -> q.eq(BaseIdPo::getId, businessId)
                .eq(PurchaseReturn::getFlowId, flowId)
                .set(PurchaseReturn::getFlowStatus, FlowStatusEnum.IN_PROGRESS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId()));
    }

    /**
     * 流程驳回
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        purchaseReturnService.update(q -> q
                .eq(BaseIdPo::getId, FlowThreadLocalUtil.getBusinessId())
                .set(PurchaseReturn::getFlowStatus, flowStatus.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        // 驳回解锁库存
        List<PurchaseReturnBom> purchaseReturnBomList = purchaseReturnBomService.list(q -> q
                .eq(PurchaseReturnBom::getPurchaseReturnId, businessId));

        Map<Long, BigDecimal> bomMap = purchaseReturnBomList.stream()
                .collect(Collectors.toMap(
                        PurchaseReturnBom::getBomSpecId,
                        PurchaseReturnBom::getReturnQuantity,
                        BigDecimal::add
                ));
        inventoryService.unlockStorage(bomMap, WarehouseConstant.PURCHASE_DEFECTIVE);
    }

    /**
     * 作废
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        reject(flowId, businessId, flowStatus);
    }

    /**
     * 验证数据
     */
    private void validated(PurchaseReturnDto purchaseReturnDto) {
        List<PurchaseReturnBom> purchaseReturnBomList = purchaseReturnDto.getPurchaseReturnBomList();
        Assert.notEmpty(purchaseReturnBomList, "采购退货bom数据不能为空");

        for (PurchaseReturnBom purchaseReturnBom : purchaseReturnBomList) {
            Assert.notNull(purchaseReturnBom.getPurchaseBomId(), "bom规格Id不能为空");
            Assert.notNull(purchaseReturnBom.getReturnQuantity(), "退货数量不能为空");
        }
    }

}
