package com.sd.business.service.sku.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.sku.dto.*;
import com.sd.business.entity.sku.po.Sku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.entity.sku.vo.*;
import com.sd.business.mapper.sku.SkuMapper;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.sku.SkuService;
import com.sd.business.service.sku.SkuSpecLinkService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;


/**
 * <p>
 * sku 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-03
 */
@Service
public class SkuServiceImpl extends ServiceImpl<SkuMapper, Sku> implements SkuService {

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private SkuSpecLinkService skuSpecLinkService;

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private ArtworkLibraryService artworkLibraryService;

    @Override
    public Page<SkuVo> getPage(SkuSelectDto dto) {
        IWrapper<Sku> wrapper = getWrapper();
        wrapper.orderByDesc("s", Sku::getId);
        wrapper.like("s", Sku::getCode, dto.getCode());
        wrapper.like("s", Sku::getName, dto.getName());
        wrapper.eq("s", Sku::getType, dto.getType());
        wrapper.eq("s", Sku::getBrand, dto.getBrand());

        Long deptId = SecurityUtils.getDeptId();
        if (Objects.equals(dto.getType(), 0) && deptId != 100L) {
            Department department = departmentService.getById(deptId);
            if (department != null) {
                if (StrUtil.isNotBlank(department.getWlnBrand())) {
                    List<String> brandList = Arrays.asList(department.getWlnBrand().split(","));
                    wrapper.in("s", Sku::getBrand, brandList);
                } else {
                    return new Page<>();
                }
            }
        }

        if (StrUtil.isNotBlank(dto.getSpecCode()) || StrUtil.isNotBlank(dto.getSpecName())) {
            List<Long> skuIdList = skuSpecService.listObject(SkuSpec::getSkuId, q -> q
                    .like(StrUtil.isNotBlank(dto.getSpecCode()), SkuSpec::getCode, dto.getSpecCode())
                    .like(StrUtil.isNotBlank(dto.getSpecName()), SkuSpec::getName, dto.getSpecName()));
            if (skuIdList.isEmpty()) {
                return new Page<>();
            }
            wrapper.in("s", Sku::getId, skuIdList);
        }

        Page<SkuVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<SkuVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        List<Long> skuIdList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<SkuSpec>> skuSpecMap = skuSpecService.mapKGroup(SkuSpec::getSkuId, q -> q
                .in(SkuSpec::getSkuId, skuIdList)
                .like(StrUtil.isNotBlank(dto.getSpecCode()), SkuSpec::getCode, dto.getSpecCode())
                .like(StrUtil.isNotBlank(dto.getSpecName()), SkuSpec::getName, dto.getSpecName()));

        records.forEach(item -> item.setSkuSpecList(skuSpecMap.getOrDefault(item.getId(), Collections.emptyList())));
        return page;
    }

    @Override
    public SkuVo detail(Long id) {
        // sku
        Sku sku = this.getById(id);
        Assert.notNull(sku, "未通过skuId找到sku");
        SkuVo result = BeanUtil.toBean(sku, SkuVo.class);

        // sku规格
        List<SkuSpec> list = skuSpecService.list(q -> q.in(SkuSpec::getSkuId, id));
        List<SkuSpecVo> skuSpecList = BeanUtil.copyToList(list, SkuSpecVo.class);
        artworkLibraryService.attributeAssign(skuSpecList, SkuSpec::getArtworkLibraryId, (item, artworkLibrary) -> {
            item.setDesignImgUrl(artworkLibrary.getImgUrl());
            item.setSharedFolder(artworkLibrary.getFileUrl());
        });
        result.setSkuSpecList(skuSpecList);

        // sku规格关联
        List<SkuSpecLink> skuSpecLinkList = skuSpecLinkService.list(q -> q.eq(SkuSpecLink::getSkuId, id));

        // sku规格关联map
        Map<Long, Map<Integer, List<SkuSpecLink>>> map = skuSpecLinkList.stream().collect(
                Collectors.groupingBy(SkuSpecLink::getSkuSpecId, Collectors.groupingBy(SkuSpecLink::getType)));

        // bom规格id列表
        List<Long> bomSpecIdList = new ArrayList<>();
        bomSpecIdList.addAll(list.stream().map(SkuSpec::getBomSpecId).filter(ObjectUtil::isNotNull).collect(Collectors.toList()));
        bomSpecIdList.addAll(skuSpecLinkList.stream().map(SkuSpecLink::getBomSpecId).collect(Collectors.toList()));
        Map<Long, BomSpec> bomSpecMap = new HashMap<>();

        if (!bomSpecIdList.isEmpty()) {
            bomSpecMap = bomSpecService.mapKEntity(BaseIdPo::getId, q -> q.in(BaseIdPo::getId, bomSpecIdList));
        }

        for (SkuSpecVo item : skuSpecList) {

            // 赋值主材bom名称
            Long bomSpecId = item.getBomSpecId();
            if (bomSpecId != null) {
                BomSpec bomSpec = bomSpecMap.get(bomSpecId);
                if (bomSpec != null) {
                    item.setBomSpecCode(bomSpec.getCode());
                    item.setBomSpecName(bomSpec.getName());
                }
            }

            // 包材，快递包装
            Map<Integer, List<SkuSpecLink>> typeSkuSpecLinkMap = map.get(item.getId());
            if (typeSkuSpecLinkMap == null) {
                item.setPackagingMaterialList(Collections.emptyList());
                item.setExpressPackingList(Collections.emptyList());
                continue;
            }

            // 赋值包材
            List<SkuSpecLinkVo> packagingMaterialList = createSkuSpecLinkVoList(typeSkuSpecLinkMap.get(1), bomSpecMap);
            item.setPackagingMaterialList(packagingMaterialList);

            // 赋值快递包装
            List<SkuSpecLinkVo> expressPackingList = createSkuSpecLinkVoList(typeSkuSpecLinkMap.get(2), bomSpecMap);
            item.setExpressPackingList(expressPackingList);

        }

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(SkuDto skuDto) {

        skuDto.setSource(1);
        save(skuDto);

        List<SkuSpec> skuSpecList = skuDto.getSkuSpecList().stream()
                .peek(item -> item.setSkuId(skuDto.getId()))
                .map(item -> (SkuSpec) item)
                .collect(Collectors.toList());
        skuSpecService.saveBatch(skuSpecList);

        List<SkuSpecLink> skuSpecLinkList = createSkuSpecLink(skuDto.getSkuSpecList());
        skuSpecLinkService.saveBatch(skuSpecLinkList);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(SkuDto skuDto) {

        Long skuId = skuDto.getId();
        List<SkuSpecDto> skuSpecDtoList = skuDto.getSkuSpecList();

        // 编辑sku
        updateById(skuDto);

        // 编辑sku规格
        editSkuSpecList(skuId, skuSpecDtoList);

        // 编辑sku规格关联快递和包材
        editSkuSpecLinkList(skuId, skuSpecDtoList);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Long id) {
        removeById(id);
        skuSpecService.remove(q -> q.eq(SkuSpec::getSkuId, id));
        skuSpecLinkService.remove(q -> q.eq(SkuSpecLink::getSkuId, id));
    }

    @Override
    public Page<ReplacePrincipalMaterialVo> getReplacePrincipalMaterialPage(GetReplaceMaterialListDto dto) {
        IWrapper<Object> wrapper = getReplaceWrapper(dto);
        return baseMapper.getReplacePrincipalMaterialPage(dto.getPage(), wrapper);
    }

    @Override
    public void replacePrincipalMaterial(ReplaceMaterialDto dto) {

        if (ObjectUtil.isAllEmpty(dto.getSkuCode(), dto.getSkuSpecCode(), dto.getBomSpecCode(),
                dto.getWidth(), dto.getHeight(), dto.getLength())) {
            throw new ServiceException("搜索条件不能全为空");
        }

        IWrapper<Object> wrapper = getReplaceWrapper(dto);

        List<Long> skuSpecIdList = baseMapper.selectReplaceSkuSpecIdList(wrapper);

        List<SkuSpec> skuSpecList = skuSpecIdList.stream().map(item -> {
            SkuSpec skuSpec = new SkuSpec();
            skuSpec.setId(item);
            skuSpec.setBomSpecId(dto.getReplaceBomSpecId());
            return skuSpec;
        }).collect(Collectors.toList());

        skuSpecService.updateBatchById(skuSpecList);

    }

    @Override
    public Page<ReplacePackagingMaterialVo> getReplacePackagingMaterialPage(GetReplaceMaterialListDto dto) {

        IWrapper<Object> wrapper = IWrapper.getWrapper()
                .eq("sslt", SkuSpecLink::getType, 1)
                .eq("s", Sku::getCode, dto.getSkuCode())
                .like("ss", SkuSpec::getCode, dto.getSkuSpecCode())
                .eq("bs", BomSpec::getCode, dto.getBomSpecCode())
                .eq("ss", SkuSpec::getLength, dto.getLength())
                .eq("ss", SkuSpec::getWidth, dto.getWidth())
                .eq("ss", SkuSpec::getHeight, dto.getHeight());

        Page<ReplacePackagingMaterialVo> page = baseMapper.getReplacePackagingMaterialPage(dto.getPage(), wrapper);
        List<ReplacePackagingMaterialVo> records = page.getRecords();

        if (records.isEmpty()) {
            return page;
        }

        List<Long> skuSpecIdList = records.stream().map(ReplacePackagingMaterialVo::getSkuSpecId).collect(Collectors.toList());

        List<SkuSpecLink> skuSpecLinkList = skuSpecLinkService.list(q -> q
                .in(SkuSpecLink::getSkuSpecId, skuSpecIdList)
                .eq(SkuSpecLink::getType, 1));

        Map<Long, List<SkuSpecLink>> map = skuSpecLinkList.stream().collect(Collectors.groupingBy(SkuSpecLink::getSkuSpecId));

        List<Long> bomSpecIdList = skuSpecLinkList.stream().map(SkuSpecLink::getBomSpecId).collect(Collectors.toList());

        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        for (ReplacePackagingMaterialVo record : records) {
            Long skuSpecId = record.getSkuSpecId();
            List<SkuSpecLink> tempSkuSpecLinkList = map.get(skuSpecId);
            if (ObjectUtil.isEmpty(tempSkuSpecLinkList)) {
                continue;
            }

            for (SkuSpecLink skuSpecLink : tempSkuSpecLinkList) {
                Long bomSpecId = skuSpecLink.getBomSpecId();
                BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
                if (bomSpecBo == null) {
                    break;
                }
                Long classifyId = bomSpecBo.getClassifyId();
                String bomSpecCode = bomSpecBo.getBomSpecCode();

                if (Objects.equals(classifyId, 1682221201491058690L)) {
                    record.setColouredPaper(record.getColouredPaper() == null ? bomSpecCode : record.getColouredPaper() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221249268375554L)) {
                    record.setOppMembrane(record.getOppMembrane() == null ? bomSpecCode : record.getOppMembrane() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221303530086402L)) {
                    record.setPeBag(record.getPeBag() == null ? bomSpecCode : record.getPeBag() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221356147630081L)) {
                    record.setMeshBag(record.getMeshBag() == null ? bomSpecCode : record.getMeshBag() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221409847304194L)) {
                    record.setPaperBox(record.getPaperBox() == null ? bomSpecCode : record.getPaperBox() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221453145104386L)) {
                    record.setBubblePack(record.getBubblePack() == null ? bomSpecCode : record.getBubblePack() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221528948760578L)) {
                    record.setLogisticsPackagingMaterial(record.getLogisticsPackagingMaterial() == null ? bomSpecCode : record.getLogisticsPackagingMaterial() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221581453058049L)) {
                    record.setOtherPackingMaterial(record.getOtherPackingMaterial() == null ? bomSpecCode : record.getOtherPackingMaterial() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1697442459841290241L)) {
                    record.setSelfAdhesiveSticker(record.getSelfAdhesiveSticker() == null ? bomSpecCode : record.getSelfAdhesiveSticker() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1697442971286331393L)) {
                    record.setDrop(record.getDrop() == null ? bomSpecCode : record.getDrop() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221651040755714L)) {
                    record.setSuspenders(record.getSuspenders() == null ? bomSpecCode : record.getSuspenders() + "," + bomSpecCode);
                } else if (Objects.equals(classifyId, 1682221712801882114L)) {
                    record.setAccessory(record.getAccessory() == null ? bomSpecCode : record.getAccessory() + "," + bomSpecCode);
                }

            }
        }

        return page;
    }

    @Override
    public void replacePackagingMaterial(ReplacePackagingMaterialDto dto) {

        IWrapper<Object> wrapper = IWrapper.getWrapper()
                .eq("sslt", SkuSpecLink::getType, 1)
                .eq("s", Sku::getCode, dto.getSkuCode())
                .like("ss", SkuSpec::getCode, dto.getSkuSpecCode())
                .eq("bs", BomSpec::getCode, dto.getBomSpecCode())
                .eq("ss", SkuSpec::getLength, dto.getLength())
                .eq("ss", SkuSpec::getWidth, dto.getWidth())
                .eq("ss", SkuSpec::getHeight, dto.getHeight());

        List<ReplacePackagingMaterialVo> list = baseMapper.getReplacePackagingMaterialList(wrapper);

        if (list.isEmpty()) {
            return;
        }

        List<Long> skuSpecIdList = list.stream().map(ReplacePackagingMaterialVo::getSkuSpecId).collect(Collectors.toList());

        List<SkuSpecLink> skuSpecLinkList = skuSpecLinkService.list(q ->
                q.in(SkuSpecLink::getSkuSpecId, skuSpecIdList).eq(SkuSpecLink::getType, 1));

        Map<Long, List<SkuSpecLink>> map = skuSpecLinkList.stream().collect(Collectors.groupingBy(SkuSpecLink::getSkuSpecId));

        Long replaceBomSpecId = dto.getReplaceBomSpecId();

        List<Long> bomSpecIdList = skuSpecLinkList.stream().map(SkuSpecLink::getBomSpecId).collect(Collectors.toList());
        bomSpecIdList.add(replaceBomSpecId);

        Map<Long, BomSpecBo> bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);

        BomSpecBo replaceBomSpecBo = bomSpecBoMap.get(replaceBomSpecId);
        if (replaceBomSpecBo == null) {
            throw new ServiceException("没有找到替换包材bom规格");
        }

        List<SkuSpecLink> addOrEditSkuSpecLinkList = new ArrayList<>();
        List<Long> removeSkuSpecLinkIdList = new ArrayList<>();

        for (ReplacePackagingMaterialVo record : list) {
            Long skuSpecId = record.getSkuSpecId();
            List<SkuSpecLink> tempSkuSpecLinkList = map.get(skuSpecId);
            if (ObjectUtil.isEmpty(tempSkuSpecLinkList)) {
                SkuSpecLink skuSpecLink = new SkuSpecLink();
                skuSpecLink.setSkuId(record.getSkuId());
                skuSpecLink.setSkuSpecId(record.getSkuSpecId());
                skuSpecLink.setBomSpecId(replaceBomSpecId);
                skuSpecLink.setType(1);
                skuSpecLink.setQuantity(BigDecimal.ONE);
                addOrEditSkuSpecLinkList.add(skuSpecLink);
                continue;
            }

            boolean flag = false;
            for (SkuSpecLink skuSpecLink : tempSkuSpecLinkList) {
                Long bomSpecId = skuSpecLink.getBomSpecId();
                BomSpecBo bomSpecBo = bomSpecBoMap.get(bomSpecId);
                if (bomSpecBo == null) {
                    continue;
                }

                if (Objects.equals(bomSpecBo.getClassifyId(), replaceBomSpecBo.getClassifyId())) {
                    if (flag) {
                        removeSkuSpecLinkIdList.add(skuSpecLink.getId());
                        continue;
                    }
                    skuSpecLink.setBomSpecId(replaceBomSpecId);
                    addOrEditSkuSpecLinkList.add(skuSpecLink);
                    flag = true;
                }

            }

            if (!flag) {
                SkuSpecLink skuSpecLink = new SkuSpecLink();
                skuSpecLink.setSkuId(record.getSkuId());
                skuSpecLink.setSkuSpecId(record.getSkuSpecId());
                skuSpecLink.setBomSpecId(replaceBomSpecId);
                skuSpecLink.setType(1);
                skuSpecLink.setQuantity(BigDecimal.ONE);
                addOrEditSkuSpecLinkList.add(skuSpecLink);
            }

        }

        if (!addOrEditSkuSpecLinkList.isEmpty()) {
            skuSpecLinkService.saveOrUpdateBatch(addOrEditSkuSpecLinkList);
        }
        if (!removeSkuSpecLinkIdList.isEmpty()) {
            skuSpecLinkService.removeBatchByIds(removeSkuSpecLinkIdList);
        }

    }

    @Override
    public void removePackagingMaterial(ReplacePackagingMaterialDto dto) {

        if (StrUtil.isBlank(dto.getBomSpecCode())) {
            throw new ServiceException("bom规格不能为空");
        }

        BomSpec bomSpec = bomSpecService.getOne(q -> q.eq(BomSpec::getCode, dto.getBomSpecCode()));
        if (bomSpec == null) {
            throw new ServiceException("未找到对应的bom规格");
        }

        skuSpecLinkService.remove(q -> q
                .eq(SkuSpecLink::getBomSpecId, bomSpec.getId())
                .eq(SkuSpecLink::getType, 1));

    }

    private IWrapper<Object> getReplaceWrapper(GetReplaceMaterialListDto dto) {

        Assert.notNull(dto.getType(), "sku类型不能为空");

        return IWrapper.getWrapper()
                .eq("s", Sku::getCode, dto.getSkuCode())
                .eq("s", Sku::getType, dto.getType())
                .like("ss", SkuSpec::getCode, dto.getSkuSpecCode())
                .eq("bs", BomSpec::getCode, dto.getBomSpecCode())
                .eq("ss", SkuSpec::getLength, dto.getLength())
                .eq("ss", SkuSpec::getWidth, dto.getWidth())
                .eq("ss", SkuSpec::getHeight, dto.getHeight());

    }

    /**
     * 创建sku规格关联
     */
    private List<SkuSpecLinkVo> createSkuSpecLinkVoList(List<SkuSpecLink> skuSpecLinkList, Map<Long, BomSpec> bomSpecMap) {

        if (ObjectUtil.isEmpty(skuSpecLinkList)) {
            return Collections.emptyList();
        }

        return skuSpecLinkList.stream().map(item -> {
            SkuSpecLinkVo skuSpecLinkVo = BeanUtil.toBean(item, SkuSpecLinkVo.class);
            BomSpec bomSpec = bomSpecMap.get(item.getBomSpecId());
            if (bomSpec != null) {
                skuSpecLinkVo.setName(bomSpec.getName());
                skuSpecLinkVo.setInternalSellingPrice(bomSpec.getInternalSellingPrice());
            }
            return skuSpecLinkVo;
        }).collect(Collectors.toList());

    }

    /**
     * 编辑sku规格
     */
    private void editSkuSpecList(Long skuId, List<SkuSpecDto> skuSpecDtoList) {

        List<Long> skuSpecIdList = skuSpecDtoList.stream().map(SkuSpec::getId)
                .filter(ObjectUtil::isNotNull).collect(Collectors.toList());

        skuSpecService.remove(q -> q
                .eq(SkuSpec::getSkuId, skuId)
                .notIn(ObjectUtil.isNotEmpty(skuSpecIdList), BaseIdPo::getId, skuSpecIdList));

        List<SkuSpec> skuSpecList = skuSpecDtoList.stream()
                .peek(item -> item.setSkuId(skuId))
                .map(item -> (SkuSpec) item)
                .collect(Collectors.toList());
        skuSpecService.saveOrUpdateBatch(skuSpecList);

    }

    /**
     * 编辑sku规格关联快递和包材
     */
    private void editSkuSpecLinkList(Long skuId, List<SkuSpecDto> skuSpecDtoList) {

        List<Long> skuSpecLinkIdList = skuSpecDtoList.stream()
                .flatMap(item -> Stream.concat(
                        item.getPackagingMaterialList().stream(),
                        item.getExpressPackingList().stream()))
                .map(BaseIdPo::getId)
                .filter(ObjectUtil::isNotNull)
                .collect(Collectors.toList());

        skuSpecLinkService.remove(q -> q
                .eq(SkuSpecLink::getSkuId, skuId)
                .notIn(ObjectUtil.isNotEmpty(skuSpecLinkIdList), BaseIdPo::getId, skuSpecLinkIdList));

        List<SkuSpecLink> skuSpecLinkList = createSkuSpecLink(skuSpecDtoList);
        skuSpecLinkService.saveOrUpdateBatch(skuSpecLinkList);

    }

    /**
     * 创建sku规格关联快递和包材
     */
    private List<SkuSpecLink> createSkuSpecLink(List<SkuSpecDto> skuSpecDtoList) {

        return skuSpecDtoList.stream().flatMap(item -> {

            List<SkuSpecLink> packagingMaterialList = item.getPackagingMaterialList();
            for (SkuSpecLink skuSpecLink : packagingMaterialList) {
                skuSpecLink.setSkuId(item.getSkuId());
                skuSpecLink.setSkuSpecId(item.getId());
                skuSpecLink.setType(1);
            }

            List<SkuSpecLink> expressPackingList = item.getExpressPackingList();
            for (SkuSpecLink skuSpecLink : expressPackingList) {
                skuSpecLink.setSkuId(item.getSkuId());
                skuSpecLink.setSkuSpecId(item.getId());
                skuSpecLink.setType(2);
            }

            return Stream.concat(packagingMaterialList.stream(), expressPackingList.stream());

        }).collect(Collectors.toList());

    }

}
