package com.sd.wln.context;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.framework.mybatis.holder.LogicHolder;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.order.po.*;
import com.sd.business.entity.price.po.PriceBillingStandard;
import com.sd.business.entity.price.po.PriceBillingStandardDetail;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.entity.sku.po.SkuSpecLink;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.price.PriceBillingStandardDetailService;
import com.sd.business.service.price.PriceBillingStandardService;
import com.sd.business.service.sku.SkuSpecLinkService;
import com.sd.business.service.sku.SkuSpecService;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Slf4j
public class OrderContext {

    private static final OrderService orderService = SpringUtil.getBean(OrderService.class);
    private static final OrderSkuService orderSkuService = SpringUtil.getBean(OrderSkuService.class);
    private static final DepartmentService departmentService = SpringUtil.getBean(DepartmentService.class);
    private static final SkuSpecService skuSpecService = SpringUtil.getBean(SkuSpecService.class);
    private static final SkuSpecLinkService skuSpecLinkService = SpringUtil.getBean(SkuSpecLinkService.class);
    private static final PriceBillingStandardService priceBillingStandardService = SpringUtil.getBean(PriceBillingStandardService.class);
    private static final PriceBillingStandardDetailService priceBillingStandardDetailService = SpringUtil.getBean(PriceBillingStandardDetailService.class);
    private static final ArtworkLibraryService artworkLibraryService = SpringUtil.getBean(ArtworkLibraryService.class);

    /**
     * 保存订单列表
     */
    @Getter
    private final List<OrderInfo> saveOrderList = new ArrayList<>();

    /**
     * 更新订单列表
     */
    @Getter
    private final List<OrderInfo> updateOrderList = new ArrayList<>();

    /**
     * 需要保存对账单的订单列表
     */
    @Getter
    private final List<OrderInfo> saveStatementOrderList = new ArrayList<>();

    /**
     * 保存订单明细列表
     */
    @Getter
    private final List<OrderSku> saveOrderSkuList = new ArrayList<>();

    /**
     * 保存订单包材
     */
    @Getter
    private final List<OrderSkuBom> saveOrderSkuBomList = new ArrayList<>();

    /**
     * 保存订单产品生产成本
     */
    @Getter
    private final List<OrderSkuProductionCost> saveOrderSkuProductionCostList = new ArrayList<>();

    /**
     * 保存订单包装
     */
    @Getter
    private final List<OrderPackageBom> orderPackageBomList = new ArrayList<>();

    /**
     * 万里牛订单列表
     */
    @Getter
    private final List<JSONObject> wlnOrderList;

    /**
     * 事业部
     */
    @Getter
    private final Department department;

    /**
     * 已存在数据库中的订单map
     * key：万里牛订单uid
     * value：订单实体
     */
    private Map<String, OrderInfo> existOrderMap;

    /**
     * 已存在数据库中的订单sku map
     * key：订单id
     * value：订单sku实体列表
     */
    private Map<Long, List<OrderSku>> existOrderSkuMap;

    /**
     * sku规格map
     * key：万里牛sku规格uid
     * value：sku规格实体
     */
    private Map<String, SkuSpec> skuSpecMap;

    /**
     * sku规格关联bom
     * key：sku规格id
     * value： key：类型 1包材 2快递包装 value：sku规格实体
     */
    private Map<Long, Map<Integer, List<SkuSpecLink>>> skuSpecLinkBomSpecMap;

    /**
     * bom规格明细
     * key：bom规格id
     * value：bom规格明细
     */
    private Map<Long, BomSpecBo> bomSpecBoMap;

    /**
     * 生产文件
     */
    private Map<Long, ArtworkLibrary> artworkLibraryMap;

    /**
     * 加工计费标准明细列表
     */
    private Map<String, List<PriceBillingStandardDetail>> priceBillingStandardMap;

    /**
     * 实例化
     */
    public OrderContext(String warehouseCode, List<JSONObject> wlnOrderList) {
        this.wlnOrderList = wlnOrderList;
        this.department = departmentService.getDepartmentByWarehouseCode(warehouseCode);
    }

    /**
     * 获取已存在数据库中的订单map
     */
    public Map<String, OrderInfo> getExistOrderMap() {
        if (existOrderMap == null) {
            List<String> uidList = wlnOrderList.stream().map(item -> item.getString("uid")).collect(Collectors.toList());
            LogicHolder.setLogicHolder(true);
            existOrderMap = orderService.mapKEntity(OrderInfo::getWlnUid, q -> q.in(OrderInfo::getWlnUid, uidList));
            LogicHolder.clear();
        }
        return existOrderMap;
    }

    /**
     * 已存在数据库中的订单sku map
     */
    public Map<Long, List<OrderSku>> getExistOrderSkuMap() {
        if (existOrderSkuMap == null) {
            List<Long> orderIdList = getExistOrderMap().values().stream().map(BaseIdPo::getId).collect(Collectors.toList());
            if (orderIdList.isEmpty()) {
                existOrderSkuMap = Collections.emptyMap();
            } else {
                existOrderSkuMap = orderSkuService.mapKGroup(OrderSku::getOrderId, q -> q.in(OrderSku::getOrderId, orderIdList));
            }
        }
        return existOrderSkuMap;
    }

    /**
     * 获取sku规格map
     */
    public Map<String, SkuSpec> getSkuSpecMap() {
        if (skuSpecMap == null) {
            List<String> wlnSkuSpecUidList = wlnOrderList.stream()
                    .flatMap(item -> item.getJSONArray("orders").toJavaList(JSONObject.class).stream())
                    .map(item -> item.getString("sys_spec_uid"))
                    .filter(ObjectUtil::isNotNull)
                    .distinct()
                    .collect(Collectors.toList());

            if (wlnSkuSpecUidList.isEmpty()) {
                skuSpecMap = Collections.emptyMap();
            } else {
                skuSpecMap = skuSpecService.mapKEntity(SkuSpec::getWlnUid, q -> q.in(SkuSpec::getWlnUid, wlnSkuSpecUidList));
            }
        }
        return skuSpecMap;
    }

    /**
     * 获取sku规格关联bom
     */
    public Map<Long, Map<Integer, List<SkuSpecLink>>> getSkuSpecLinkBomSpecMap() {

        if (skuSpecLinkBomSpecMap == null) {

            if (department == null) {
                skuSpecLinkBomSpecMap = Collections.emptyMap();
                return skuSpecLinkBomSpecMap;
            }

            Collection<SkuSpec> values = getSkuSpecMap().values();
            if (values.isEmpty()) {
                skuSpecLinkBomSpecMap = Collections.emptyMap();
                return skuSpecLinkBomSpecMap;
            }

            List<Long> skuSpecIdList = values.stream().map(SkuSpec::getId).distinct().collect(Collectors.toList());
            List<SkuSpecLink> list = skuSpecLinkService.list(q -> q.in(SkuSpecLink::getSkuSpecId, skuSpecIdList));

            skuSpecLinkBomSpecMap = list.stream().collect(Collectors.groupingBy(
                    SkuSpecLink::getSkuSpecId, Collectors.groupingBy(SkuSpecLink::getType)));
        }

        return skuSpecLinkBomSpecMap;
    }

    /**
     * 获取bom规格明细
     */
    public Map<Long, BomSpecBo> getBomSpecBoMap() {
        if (bomSpecBoMap == null) {
            List<Long> bomSpecIdList = Stream.concat(
                            getSkuSpecMap().values().stream().map(SkuSpec::getBomSpecId),
                            getSkuSpecLinkBomSpecMap().values().stream().flatMap(item -> Stream.concat(
                                    item.getOrDefault(1, Collections.emptyList()).stream(),
                                    item.getOrDefault(2, Collections.emptyList()).stream()
                            )).map(SkuSpecLink::getBomSpecId))
                    .distinct().collect(Collectors.toList());
            bomSpecBoMap = skuSpecService.getBomSpecBoByIdList(bomSpecIdList);
        }
        return bomSpecBoMap;
    }

    /**
     * 获取加工计费标准明细列表
     */
    public List<PriceBillingStandardDetail> getPriceBillingStandardMap(String species) {
        if (StrUtil.isEmpty(species)) {
            return Collections.emptyList();
        }

        if (priceBillingStandardMap == null) {
            if (department == null || department.getPriceBillingStandardId() == null) {
                priceBillingStandardMap = Collections.emptyMap();
            } else {
                String priceBillingStandardId = department.getPriceBillingStandardId();
                if (StrUtil.isEmpty(priceBillingStandardId)) {
                    priceBillingStandardMap = Collections.emptyMap();
                } else {
                    priceBillingStandardMap = new HashMap<>();
                    List<Long> idList = Arrays.stream(priceBillingStandardId.split(",")).map(Convert::toLong).collect(Collectors.toList());
                    List<PriceBillingStandard> priceBillingStandardList = priceBillingStandardService.listByIds(idList);

                    Map<Long, List<PriceBillingStandardDetail>> map = priceBillingStandardDetailService.mapKGroup(
                            PriceBillingStandardDetail::getPriceBillingStandardId,
                            q -> q.in(PriceBillingStandardDetail::getPriceBillingStandardId, idList));

                    for (PriceBillingStandard priceBillingStandard : priceBillingStandardList) {
                        priceBillingStandardMap.put(priceBillingStandard.getSpecies(),
                                map.getOrDefault(priceBillingStandard.getId(), Collections.emptyList()));
                    }

                }
            }
        }
        return priceBillingStandardMap.getOrDefault(species, Collections.emptyList());
    }

    public Map<Long, ArtworkLibrary> getArtworkLibraryMap() {
        if (artworkLibraryMap != null) {
            return artworkLibraryMap;
        }

        Map<String, SkuSpec> tempSkuSpecMap = getSkuSpecMap();
        if (tempSkuSpecMap.isEmpty()) {
            artworkLibraryMap = Collections.emptyMap();
            return artworkLibraryMap;
        }

        List<Long> artworkLibraryId = tempSkuSpecMap.values().stream().map(SkuSpec::getArtworkLibraryId).collect(Collectors.toList());
        if (artworkLibraryId.isEmpty()) {
            artworkLibraryMap = Collections.emptyMap();
            return artworkLibraryMap;
        }

        List<ArtworkLibrary> artworkLibraryList = artworkLibraryService.listByIds(artworkLibraryId);
        artworkLibraryMap = artworkLibraryList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        return artworkLibraryMap;
    }
}
